import 'package:shared_preferences/shared_preferences.dart';

import 'config.dart';
import '../data/models/config.dart';
import '../data/request/native_config_request.dart';
import '../modules/liveness_detection/liveness_detection.dart';
import '../utils/file_until.dart';

class Application {
  factory Application() {
    return _instance;
  }

  Application._internal();
  static final Application _instance = Application._internal();

  static const bool debug = true;
  static const String version = '1.0.0';
  static String key = 'a08eb42a-4a57-449b-84f4-1f67219f2679';
  static const String domain = 'https://ekycdemo.smartocr.vn';
  static const String name = 'eKYC';
  static late SharedPreferences preferences;
  static late SkdConfig config;
  static void setConfig(SkdConfig skdConfig) {
    config = skdConfig;
  }

  static Future<void> init(NativeConfigRequest configModel) async {
    if (configModel.token != null && configModel.token!.isNotEmpty) {
      key = configModel.token!;
    }
    var listDetectorType = <DetectionType>[];

    if (configModel.isEnableSmile ?? true) {
      listDetectorType.add(DetectionType.SMILE);
    }
    if (configModel.isEnableLookStraight ?? true) {
      listDetectorType.add(DetectionType.LOOK_STRAIGHT);
    }
    if (configModel.isEnableLookRight ?? true) {
      listDetectorType.add(DetectionType.LOOK_RIGHT);
    }
    if (configModel.isEnableLookLeft ?? true) {
      listDetectorType.add(DetectionType.LOOK_LEFT);
    }
    if (configModel.isEnableEyesBlink ?? true) {
      listDetectorType.add(DetectionType.EYES_BLINK);
    }
    if (configModel.bg != null) {
      AppTheme.primaryColor = HexColor(configModel.bg!);
      AppTheme.onBackground = HexColor(configModel.color!);
    }
    config = SkdConfig(
      color: configModel.color == null
          ? AppTheme.primaryColor
          : HexColor(configModel.color!),
      bg: configModel.color == null
          ? AppTheme.primaryColor
          : HexColor(configModel.color!),
      isHistoy: configModel.isHistory ?? true,
      isSignature: configModel.isSignature ?? true,
      isShowInfo: configModel.isShowInfo ?? true,
      readOnlyInfo: configModel.readOnlyInfo ?? false,
      readOnlyId: configModel.readOnlyId ?? true,
      isSaveCapture: configModel.isSaveCapture ?? false,
      isAutoMatching: configModel.isAutoMatching ?? true,
      faceConfig: FaceConfig(
        listDetectorType: listDetectorType,
        doingStep: configModel.takeDoingStep ?? listDetectorType.length,
        detectionLastStep: DetectionLastStep.TAKE_PORTRAITS,
        detectionDisplay: configModel.isRandomDetect ?? true
            ? DetectionDisplay.RANDOM
            : DetectionDisplay.DEFAULT,
      ),
    );
    preferences = await SharedPreferences.getInstance();
    FileUtils.init();
  }
}

class HexColor extends Color {
  static int _getColorFromHex(String hexColor) {
    hexColor = hexColor.toUpperCase().replaceAll('#', '');
    if (hexColor.length == 6) {
      hexColor = 'FF' + hexColor;
    }
    return int.parse(hexColor, radix: 16);
  }

  HexColor(final String hexColor) : super(_getColorFromHex(hexColor));
}

class ColorToHex extends Color {
  ///convert material colors to hexcolor
  static int _convertColorTHex(Color color) {
    var hex = '${color.value}';
    return int.parse(
      hex,
    );
  }

  ColorToHex(final Color color) : super(_convertColorTHex(color));
}
