import 'package:intl/intl.dart';
import 'dart:convert';

import 'cmnd_model.dart';
import 'face_model.dart';

typedef Json = Map<String, dynamic>;

class HistoryModel {
  HistoryModel({
    required this.id,
    this.cmndModel,
    this.faceComprate,
    this.faces,
    required this.createAt,
    this.cardFront,
    this.cardBackside,
    this.portraitsImage,
    this.signature,
  });
  String id;
  CMNDModel? cmndModel;
  FaceComprate? faceComprate;
  List<String>? faces;
  String? cardFront, cardBackside;
  String? portraitsImage;
  String? signature;
  DateTime createAt;
  String get createAtString =>
      DateFormat('HH:mm - dd/MM/yyyy').format(createAt);
  bool get check =>
      cmndModel?.logiccheck == 'GOOD' &&
      faceComprate?.message == 'Same person' &&
      cmndModel?.frontBackType == 'same type' &&
      cmndModel?.logiccheck == 'GOOD' &&
      cmndModel?.blurCheck == 'GOOD' &&
      cmndModel?.colorCheck == 'GOOD' &&
      cmndModel?.cornerCheck == 'GOOD' &&
      cmndModel?.expireCheck == 'OK' &&
      cmndModel?.pasteOverCheck == 'GOOD' &&
      cmndModel?.throwScreenCheck == 'GOOD';
  factory HistoryModel.fromMap(String string) {
    var item = json.decode(string);
    return HistoryModel(
      id: item['id'] ?? DateTime.now().microsecondsSinceEpoch.toString(),
      cmndModel: item['cmndModel'] == null
          ? null
          : CMNDModel.fromMap(item['cmndModel']),
      faceComprate: item['faceComprate'] == null
          ? null
          : FaceComprate.fromMap(item['faceComprate']),
      faces: item['faces'] == null
          ? null
          : List<dynamic>.from(item['faces']).map((e) => e.toString()).toList(),
      createAt: DateTime.fromMillisecondsSinceEpoch(item['createAt']),
      cardFront: item['cardFront'] ?? '',
      cardBackside: item['cardBackside'] ?? '',
      portraitsImage: item['portraitsImage'] ?? '',
      signature: item['signature'] ?? '',
    );
  }

  Json toMap() => {
        'cmndModel': cmndModel != null ? cmndModel!.toMap() : null,
        'faceComprate': faceComprate != null ? faceComprate!.toMap() : null,
        'faces': faces,
        'createAt': createAt.millisecondsSinceEpoch,
        'cardFront': cardFront,
        'cardBackside': cardBackside,
        'portraitsImage': portraitsImage,
        'signature': signature,
        'id': id,
      };
  String toJson() => json.encode(toMap());
}
