import 'dart:typed_data';
import 'package:flutter/material.dart';
import 'package:get/get.dart' hide ContextExtensionss;

import '../../../utils/app_utils.dart';
import '../../../configs/config.dart';
import '../../../widgets/widget.dart';

import '../controllers/display_photo_controller.dart';

class DisplayPhotoViewArguments {
  final bool isBackSide;
  final Uint8List image;
  final String btnText;

  DisplayPhotoViewArguments({
    required this.isBackSide,
    required this.image,
    required this.btnText,
  });
}

class DisplayPhotoView extends GetView<DisplayPhotoController> {
  const DisplayPhotoView({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    final size = Size(
      context.width * .8,
      (context.width * .8) / (5 / 3.5),
    );
    return WillPopScope(
      onWillPop: AppUtils.onWillPop,
      child: Scaffold(
        backgroundColor: context.primaryVariant,
        body: SafeArea(
          child: [
            Expanded(
              child: Image.memory(controller.arguments.image, fit: BoxFit.cover)
                  .box
                  .size(size.width, size.height)
                  .make()
                  .centered(),
            ),
            10.heightBox,
            'take_again'
                .tr
                .text
                .underline
                .bold
                .color(Vx.white)
                .size(18)
                .makeCentered()
                .onTap(controller.retakePicture),
            20.heightBox,
            AppButton(
              controller.arguments.btnText,
              borderRadius: 30,
              fontSize: 18,
              axisSize: MainAxisSize.min,
              onPressed: () {
                controller.scanBackSide(controller.arguments.isBackSide);
              },
            )
                .box
                .height(50)
                .withConstraints(const BoxConstraints(minWidth: 220))
                .make(),
            20.heightBox,
          ].vStack(),
        ),
      ),
    );
  }
}
