import 'package:flutter/material.dart';
import 'package:get/get.dart' hide ContextExtensionss;

import '../../../configs/application.dart';
import '../../../configs/config.dart';
import '../../../routes/app_pages.dart';
import '../../../widgets/app_button.dart';
import '../controllers/history_controller.dart';

class ChooseScreen extends StatelessWidget {
  const ChooseScreen({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        backgroundColor: Colors.white,
        appBar: AppBar(
          backgroundColor: Colors.transparent,
          elevation: 0,
          title: Dimens.logo,
          centerTitle: true,
        ),
        body: SafeArea(
          child: Padding(
            padding: const EdgeInsets.symmetric(
              vertical: 30,
            ),
            child: Column(
              children: [
                'choose action'.tr.text.size(24).bold.make(),
                (context.height * .1).heightBox,
                Row(
                  mainAxisAlignment: MainAxisAlignment.center,
                  children: [
                    _item(
                        title: 'New eKYC'.tr,
                        image: AppImage.choose1,
                        onTap: (() {
                          Get.offNamed(Routes.NOTE);
                        })),
                    if (Application.config.isHistoy)
                      _item(
                        title: 'history'.tr + '\n',
                        image: AppImage.choose2,
                        onTap: (() {
                          Get.toNamed(Routes.HISTORY);
                        }),
                      ),
                  ],
                )
              ],
            ),
          ),
        ));
  }

  Widget _item({
    required String title,
    required String image,
    required VoidCallback onTap,
  }) {
    return [
      SizedBox(
        width: 150,
        height: 100,
        child: Image.asset(image),
      ),
      Dimens.height10,
      title.text.size(18).make(),
    ].column().onInkTap(onTap);
  }
}

class HistoryView extends GetView<HistoryController> {
  const HistoryView({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: Colors.grey[200],
      appBar: AppBar(
        elevation: 0,
        automaticallyImplyLeading: false,
        title: Row(
          children: [
            Icon(
              Icons.arrow_back_ios,
              color: context.onBackground,
            ).p(5).onInkTap(() => Get.back()),
            Dimens.width10,
            'history'.tr.text.size(18).bold.make(),
          ],
        ),
      ),
      body: Column(
        children: [
          Dimens.height10,
          Obx(
            () => [
              AppButton(
                'all'.tr,
                elevation: 2,
                color: controller.fillter.value == HistoryType.all
                    ? context.primaryColor
                    : context.onBackground,
                textColor: controller.fillter.value == HistoryType.all
                    ? Colors.white
                    : Colors.black,
                onPressed: () => controller.filter(HistoryType.all),
              ),
              Dimens.width5,
              AppButton(
                'valid'.tr,
                elevation: 2,
                icon: Image.asset(AppImage.checked, width: 24),
                color: controller.fillter.value == HistoryType.valid
                    ? context.primaryColor
                    : Colors.white,
                textColor: controller.fillter.value == HistoryType.valid
                    ? Colors.white
                    : Colors.black,
                onPressed: () => controller.filter(HistoryType.valid),
              ),
              Dimens.width5,
              AppButton(
                'not_valid'.tr,
                elevation: 2,
                icon: Image.asset(AppImage.cancel, width: 24),
                color: controller.fillter.value == HistoryType.novalid
                    ? context.primaryColor
                    : Colors.white,
                textColor: controller.fillter.value == HistoryType.novalid
                    ? Colors.white
                    : Colors.black,
                onPressed: () => controller.filter(HistoryType.novalid),
              ),
            ]
                .row(
                  axisSize: MainAxisSize.max,
                  alignment: MainAxisAlignment.spaceBetween,
                )
                .pSymmetric(h: 20),
          ),
          Dimens.height20,
          Expanded(
            child: controller.obx(
              (state) {
                var items = List.from(state!).reversed.toList();
                return ListView.builder(
                  padding: const EdgeInsets.symmetric(horizontal: 15),
                  itemCount: items.length,
                  itemBuilder: (BuildContext context, int index) {
                    final item = items[index];
                    return Card(
                      child: [
                        Image.asset(
                            item.check ? AppImage.checked : AppImage.cancel,
                            width: 26,
                            height: 26),
                        Dimens.width10,
                        [
                          '${'date_time'.tr} ${item.createAtString}'
                              .text
                              .size(18)
                              .make(),
                          Dimens.height5,
                          _text('number'.tr + ': ', item.cmndModel!.id!),
                          Dimens.height5,
                          _text('full_name'.tr + ': ', item.cmndModel!.name!),
                          // Row(children: item.faces!.map((e) => ImageThumbnail(path: e)).toList())
                        ].column(
                          crossAlignment: CrossAxisAlignment.start,
                        )
                      ]
                          .row()
                          .box
                          .padding(const EdgeInsets.all(10))
                          .make()
                          .onInkTap(() {
                        Get.toNamed(Routes.INFOMATION, arguments: item);
                      }),
                    );
                  },
                );
              },
            ),
          ),
        ],
      ).safeArea(),
    );
  }

  Widget _text(String? title, String? value) {
    return RichText(
      text: TextSpan(
        children: [
          TextSpan(
            text: title ?? '',
            style: const TextStyle(
              fontWeight: FontWeight.bold,
              color: Colors.black,
              fontSize: 18,
            ),
          ),
          TextSpan(
            text: value ?? '',
            style: const TextStyle(
              color: Colors.black,
              fontSize: 18,
            ),
          ),
        ],
      ),
    );
  }
}
