import 'package:get/get.dart' hide ContextExtensionss;
import 'package:flutter/material.dart';
import 'package:flutter/cupertino.dart';

import '../../../configs/config.dart';
import '../../../configs/application.dart';
import '../../../utils/app_utils.dart';
import '../../../widgets/widget.dart';
import '../controllers/id_card_scanner_controller.dart';

class IdCardScannerViewArguments {
  final String title;
  final bool isBackSide;

  IdCardScannerViewArguments({required this.title, required this.isBackSide});
}

class IdCardScannerView extends GetView<IdCardScannerController> {
  const IdCardScannerView({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    final IdCardScannerViewArguments arguments = Get.arguments ??
        IdCardScannerViewArguments(
            title: 'front'.tr.toLowerCase(), isBackSide: false);
    final size = Size(
      context.width * .8,
      (context.width * .8) / (5 / 3.5),
    );
    return WillPopScope(
      onWillPop: AppUtils.onWillPop,
      child: Scaffold(
        key: controller.scaffoldKey,
        body: SizedBox(
          width: context.width,
          height: context.height,
          child: Stack(
            children: [
              GetBuilder<IdCardScannerController>(
                builder: (_) {
                  if (_.cameraInitial) {
                    return [
                      CameraPreview(controller.cameraCtrller)
                          .w(double.infinity)
                          .centered(),
                      Align(
                        alignment: Alignment.center,
                        child: SizedBox(
                          width: size.width,
                          height: size.height,
                          key: controller.imageKey,
                          child: CustomPaint(
                            size: size,
                            painter: MarkerCanvas(
                              size: size,
                              color: context.onBackground,
                            ),
                          ),
                        ),
                      ),
                    ].zStack();
                  } else {
                    return Container(
                      color: Vx.white,
                      child: const Center(
                        child: CircularProgressIndicator(),
                      ),
                    );
                  }
                },
              ),
              Align(
                alignment: Alignment.topCenter,
                child: [
                  65.heightBox,
                  'please_give'
                      .tr
                      .richText
                      .withTextSpanChildren([
                        arguments.title.textSpan.size(24).bold.underline.make(),
                        'of_id_card'.tr.textSpan.make(),
                      ])
                      .color(context.onBackground)
                      .make(),
                  35.heightBox,
                ]
                    .vStack()
                    .box
                    .color(context.primaryVariant)
                    .make()
                    .wFull(context),
              ),
              Align(
                alignment: Alignment.bottomCenter,
                child: [
                  (context.height * .05).heightBox,
                  Obx(() {
                    return (controller.loading.value
                            ? const Loading()
                            : VxBox(
                                    child: '${controller.duration.value}s'
                                        .text
                                        .size(18)
                                        .bold
                                        .color(context.onBackground)
                                        .makeCentered())
                                .size(double.infinity, double.infinity)
                                .color(Application.config.autoORC
                                    ? context.primaryVariant
                                    : context.onBackground)
                                .roundedFull
                                .make())
                        .box
                        .padding(const EdgeInsets.all(5))
                        .size(80, 80)
                        .withDecoration(
                          BoxDecoration(
                            shape: BoxShape.circle,
                            border: Border.all(
                                color: context.onBackground, width: 3),
                          ),
                        )
                        .make()
                        .onTap(() {
                      if (controller.loading.value) return;
                      controller.takePhoto(arguments.isBackSide);
                    });
                  }),
                  (context.height * .05).heightBox,
                ]
                    .vStack()
                    .box
                    .color(context.primaryVariant)
                    .make()
                    .wFull(context),
              ),
            ],
          ),
        ),
      ),
    );
  }
}

class MarkerCanvas extends CustomPainter {
  MarkerCanvas({required this.size, this.color = Colors.white}) : super();

  final Color color;
  final Size size;
  final Radius radius = const Radius.circular(8.0);
  final double lineWidth = 3;
  final double lineSize = 50;
  @override
  void paint(Canvas canvas, Size size) {
    var marker = Paint()
      ..color = color
      ..strokeWidth = 0.1;

    //corners left-top
    canvas.drawRRect(
      RRect.fromRectAndCorners(
        Rect.fromLTWH(0.0, 0.0, lineWidth, lineSize),
        topLeft: radius,
        bottomLeft: radius,
        bottomRight: radius,
        topRight: radius,
      ),
      marker,
    );

    canvas.drawRRect(
      RRect.fromRectAndCorners(
        Rect.fromLTWH(0.0, 0.0, lineSize, lineWidth),
        topLeft: radius,
        bottomLeft: radius,
        bottomRight: radius,
        topRight: radius,
      ),
      marker,
    );

    //corners right top
    canvas.drawRRect(
      RRect.fromRectAndCorners(
        Rect.fromLTWH(size.width, 0.0, lineWidth, lineSize),
        topLeft: radius,
        bottomLeft: radius,
        bottomRight: radius,
        topRight: radius,
      ),
      marker,
    );

    canvas.drawRRect(
      RRect.fromRectAndCorners(
        Rect.fromLTWH(
            size.width - lineSize + lineWidth, 0.0, lineSize, lineWidth),
        topLeft: radius,
        bottomLeft: radius,
        bottomRight: radius,
        topRight: radius,
      ),
      marker,
    );

    // corners left bottom
    canvas.drawRRect(
      RRect.fromRectAndCorners(
        Rect.fromLTWH(0.0, size.height - lineSize, lineWidth, lineSize),
        topLeft: radius,
        bottomLeft: radius,
        bottomRight: radius,
        topRight: radius,
      ),
      marker,
    );

    canvas.drawRRect(
      RRect.fromRectAndCorners(
        Rect.fromLTWH(0.0, size.height - lineWidth, lineSize, lineWidth),
        topLeft: radius,
        bottomLeft: radius,
        bottomRight: radius,
        topRight: radius,
      ),
      marker,
    );

    //corners right bottom
    canvas.drawRRect(
      RRect.fromRectAndCorners(
        Rect.fromLTWH(size.width - lineWidth, size.height - lineSize, lineWidth,
            lineSize),
        topLeft: radius,
        bottomLeft: radius,
        bottomRight: radius,
        topRight: radius,
      ),
      marker,
    );

    canvas.drawRRect(
      RRect.fromRectAndCorners(
        Rect.fromLTWH(size.width - lineSize, size.height - lineWidth, lineSize,
            lineWidth),
        topLeft: radius,
        bottomLeft: radius,
        bottomRight: radius,
        topRight: radius,
      ),
      marker,
    );
  }

  @override
  bool shouldRepaint(CustomPainter oldDelegate) {
    return false;
  }
}
