import 'dart:convert';
import 'dart:typed_data';
import 'package:get/get.dart';
import 'package:pedantic/pedantic.dart';

import '../../../data/models/face_model.dart';
import '../../../data/models/history.dart';
import '../../../data/providers/api_provider.dart';
import '../../app/controllers/app_controller.dart';
import '../../../utils/app_utils.dart';
import '../../../utils/convert_camera_image_to_png.dart';
import '../../../utils/file_until.dart';
import '../../../utils/logger.dart';

class InfomationController extends GetxController {
  AppController get to => Get.find();
  HistoryModel? historyModel;

  late FaceComprate face;
  bool loading = true;
  @override
  void onInit() {
    Future.delayed(const Duration(milliseconds: 300), () {
      init();
    });
    super.onInit();
  }

  Future<void> init() async {
    if (Get.arguments != null) {
      historyModel = Get.arguments as HistoryModel;
      loading = false;
      update();
    } else {
      final _cardFront = await FileUtils.saveByteFile(to.cardFront!);
      final _cardBackside = await FileUtils.saveByteFile(to.cardBackside!);
      final _portraitsImage = await FileUtils.saveByteFile(to.portraitsImage);
      final _signature = await FileUtils.saveByteFile(to.signature);
      historyModel = HistoryModel(
        id: DateTime.now().microsecondsSinceEpoch.toString(),
        createAt: DateTime.now(),
        cmndModel: Get.find<AppController>().cmndModel,
        cardBackside: _cardBackside,
        cardFront: _cardFront,
        portraitsImage: _portraitsImage,
        signature: _signature,
      );
      update();
      unawaited(faceComprace());
    }
  }

  void updateInfo() {
    Get.find<AppController>().updateInfo(historyModel!);
  }

  @override
  void onClose() {}
  Future<void> faceComprace() async {
    try {
      final to = Get.find<AppController>();
      if (to.imageList.isNotEmpty) {
        final response = await ApiProvider.faceCompare(
          base64.decode(to.cmndModel.imageRoi ?? ''),
          to.portraitsImage!,
        );
        if (response.statusCode == 200) {
          face = FaceComprate.fromMap(response.data);
          historyModel!.faceComprate = face;
          saveHistory();
          update();
        } else {
          saveHistory();
        }
      }
      loading = false;
      update();
    } catch (e) {
      loading = false;
      UtilLogger.snackbar(e.toString());
      UtilLogger.log(e);
    }
  }

  void saveHistory() {
    unawaited(Get.find<AppController>().addHistory(historyModel!));
    Get.find<AppController>().imageList.forEach((element) {
      var uint8list = Uint8List.fromList(convertImagetoPng(element)!);
      AppUtils.saveImage(uint8list);
    });
    if (historyModel?.cmndModel?.imageRoi != null) {
      AppUtils.saveImage(
          base64.decode(historyModel?.cmndModel?.imageRoi ?? ''));
    }
  }
}
