import 'dart:convert';
import 'dart:io';
import 'dart:typed_data';
import 'package:get/get.dart' hide ContextExtensionss;
import 'package:photo_view/photo_view.dart';

import '../../../configs/application.dart';
import '../../../configs/config.dart';
import '../../../data/models/history.dart';
import 'result_notity.dart';
import '../widgets/info_item_update.dart';
import '../../../routes/app_pages.dart';
import '../../../utils/app_utils.dart';
import '../../../widgets/app_button.dart';
import '../../../widgets/widget.dart';
import '../controllers/infomation_controller.dart';

class InfomationView extends GetView<InfomationController> {
  const InfomationView({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return WillPopScope(
      onWillPop: Get.arguments == null ? AppUtils.onWillPop : null,
      child: Scaffold(
        body: GetBuilder<InfomationController>(builder: (_) {
          if (_.loading) {
            return const Loading();
          }
          if (Application.config.isShowInfo) {
            return <Widget>[
              const _Header(),
              _.historyModel?.cmndModel != null
                  ? _Info(history: _.historyModel!)
                  : const Loading(),
              _Identification(history: _.historyModel),
              if (Application.config.isSignature &&
                  _.historyModel?.signature != null)
                _Signture(signature: _.historyModel!.signature!),
              if (Get.arguments == null)
                <Widget>[
                  AppButton(
                    'again'.tr,
                    type: ButtonType.outline,
                    textColor: Colors.white,
                    borderColor: Colors.white,
                    borderRadius: 30,
                    fontSize: 18,
                    onPressed: () {
                      Get.offAllNamed(Routes.ID_CARD_SCANNER);
                    },
                  ).box.height(50).make().expand(),
                  Dimens.width10,
                  AppButton(
                    'confirm'.tr,
                    borderRadius: 30,
                    fontSize: 18,
                    onPressed: () {
                      Get.offAllNamed(Routes.CHOOSE_SCREEN);
                    },
                  ).box.height(50).make().expand()
                ].row().p(10)
              else
                AppButton(
                  'back'.tr,
                  borderRadius: 30,
                  fontSize: 18,
                  type: ButtonType.outline,
                  textColor: Colors.white,
                  borderColor: Colors.white,
                  onPressed: (() {
                    Get.back();
                  }),
                ).box.size(220, 50).make().p(10)
            ].column().scrollVertical().safeArea();
          } else {
            return ResultNotify(result: _.historyModel!.check);
          }
        }),
      ),
    );
  }
}

class _Header extends StatelessWidget {
  const _Header({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return [
      const Icon(Icons.arrow_back_ios, color: Colors.white).p(5).onInkTap(() {
        if (Get.arguments == null) {
          AppUtils.onWillPop();
        } else {
          Get.back();
        }
      }),
      'credentials'.tr.text.bold.minFontSize(18).white.make(),
      const Spacer(),
      'check_result'
          .tr
          .text
          .white
          .make()
          .box
          .withRounded()
          .padding(const EdgeInsets.fromLTRB(12, 6, 12, 6))
          .border(color: Colors.white)
          .make()
          .onTap(() {
        Get.toNamed(Routes.RESULT);
      }),
    ]
        .row(
          alignment: MainAxisAlignment.spaceBetween,
          axisSize: MainAxisSize.max,
        )
        .p(10);
  }
}

class _Info extends StatelessWidget {
  const _Info({Key? key, required this.history}) : super(key: key);
  final HistoryModel history;
  InfomationController get to => Get.find();
  @override
  Widget build(BuildContext context) {
    return _CardItem(
      child: [
        Row(
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            Image.asset(
              history.check ? AppImage.checked : AppImage.cancel,
              width: 24,
            ),
            Dimens.width10,
            (history.check ? 'result_success' : 'result_error')
                .tr
                .text
                .size(18)
                .color(history.check ? Colors.green : Colors.red)
                .bold
                .make(),
          ],
        ),
        Dimens.height20,
        Row(
          mainAxisAlignment: MainAxisAlignment.spaceAround,
          children: [
            _item(
                title: 'CMND/CCCD',
                image: history.cmndModel!.imageRoi == null
                    ? Dimens.empty
                    : Image.memory(base64.decode(history.cmndModel!.imageRoi!)),
                onTap: () {
                  if (history.cmndModel!.imageRoi != null) {
                    Get.to(
                      () => _ZoomImage(
                        image: base64.decode(history.cmndModel!.imageRoi!),
                        tag: 'CMND/CCCD',
                      ),
                    );
                  }
                }),
            _item(
                title: 'portrait'.tr,
                image: Image.file(history.portraitsImage!.file),
                onTap: () {
                  Get.to(
                    () => _ZoomImage(
                      image: history.portraitsImage!.bytes,
                      tag: 'Chân dung',
                    ),
                  );
                }),
          ],
        ),
        InfoItemUpdate(
          title: 'number'.tr,
          readOnly: Application.config.readOnlyId,
          content: history.cmndModel!.id,
        ),
        InfoItemUpdate(
          title: 'full_name'.tr,
          content: history.cmndModel!.name,
          readOnly: Application.config.readOnlyInfo,
          innerTextChange: (String text) {
            history.cmndModel!.name = text;
            to.updateInfo();
          },
        ),
        InfoItemUpdate(
          title: 'gender'.tr,
          content: history.cmndModel!.sex,
          readOnly: Application.config.readOnlyInfo,
          innerTextChange: (String text) {
            history.cmndModel!.sex = text;
            to.updateInfo();
          },
        ),
        InfoItemUpdate(
          title: 'birthday'.tr,
          content: history.cmndModel!.birthday,
          readOnly: Application.config.readOnlyInfo,
          innerTextChange: (String text) {
            history.cmndModel!.birthday = text;
            to.updateInfo();
          },
        ),
        InfoItemUpdate(
          title: 'Home town'.tr,
          content: history.cmndModel!.homeTown,
          readOnly: Application.config.readOnlyInfo,
          innerTextChange: (String text) {
            history.cmndModel!.homeTown = text;
            to.updateInfo();
          },
        ),
        InfoItemUpdate(
          title: 'resident'.tr,
          content: history.cmndModel!.address,
          readOnly: Application.config.readOnlyInfo,
          innerTextChange: (String text) {
            history.cmndModel!.address = text;
            to.updateInfo();
          },
        ),
        InfoItemUpdate(
          title: 'use_value'.tr,
          content: history.cmndModel!.expiry,
          readOnly: Application.config.readOnlyInfo,
          innerTextChange: (String text) {
            history.cmndModel!.expiry = text;
            to.updateInfo();
          },
        ),
        InfoItemUpdate(
          title: 'issue_date'.tr,
          content: history.cmndModel!.issueDate,
          readOnly: Application.config.readOnlyInfo,
          innerTextChange: (String text) {
            history.cmndModel!.issueDate = text;
            to.updateInfo();
          },
        ),
        InfoItemUpdate(
          title: 'issue_at'.tr,
          content: history.cmndModel!.issueAt,
          readOnly: Application.config.readOnlyInfo,
          innerTextChange: (String text) {
            history.cmndModel!.issueAt = text;
            to.updateInfo();
          },
        ),
        InfoItemUpdate(
          title: 'Ethnic'.tr,
          content: history.cmndModel!.ethnicity,
          readOnly: Application.config.readOnlyInfo,
          innerTextChange: (String text) {
            history.cmndModel!.ethnicity = text;
            to.updateInfo();
          },
        ),
        InfoItemUpdate(
          title: 'Religion'.tr,
          content: history.cmndModel!.religion,
          readOnly: Application.config.readOnlyInfo,
          innerTextChange: (String text) {
            history.cmndModel!.religion = text;
            to.updateInfo();
          },
        ),
      ].column(),
    );
  }

  Widget _item({
    required String title,
    required Widget image,
    required VoidCallback onTap,
  }) {
    return [
      Hero(tag: title, child: image)
          .box
          .size(Get.width * .3, Get.width * .35)
          .red100
          .make(),
      Dimens.height10,
      title.text.size(18).make(),
    ].column().onInkTap(onTap);
  }
}

class _Identification extends StatelessWidget {
  const _Identification({Key? key, required this.history}) : super(key: key);
  final HistoryModel? history;
  @override
  Widget build(BuildContext context) {
    Widget _item(String image, String title) {
      return Expanded(
        child: Column(
          children: [
            Hero(
              tag: title,
              child: Image.file(image.file).onTap(
                () {
                  Get.to(_ZoomImage(image: image.bytes, tag: title));
                },
              ),
            ),
            Dimens.height10,
            title.tr.text.size(18).make(),
          ],
        ),
      );
    }

    return history?.cardFront != null && history?.cardBackside != null
        ? _CardItem(
            child: Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              'info_ID'
                  .tr
                  .text
                  .size(18)
                  .bold
                  .color(context.primaryColor)
                  .make(),
              Dimens.height20,
              [
                _item(history!.cardFront!, 'front'),
                Dimens.width10,
                _item(history!.cardBackside!, 'backside'),
              ].row(),
            ],
          ))
        : const SizedBox();
  }
}

class _Signture extends StatelessWidget {
  const _Signture({Key? key, required this.signature}) : super(key: key);
  final String signature;
  @override
  Widget build(BuildContext context) {
    return _CardItem(
      child: Column(
        children: [
          'signature'.tr.text.size(18).bold.color(context.primaryColor).make(),
          Dimens.height20,
          Hero(
            tag: 'signature',
            child: Image.file(signature.file).centered().onTap(() {
              Get.to(_ZoomImage(image: signature.bytes, tag: 'signature'));
            }),
          ),
        ],
      ),
    );
  }
}

class _ZoomImage extends StatelessWidget {
  const _ZoomImage({Key? key, required this.image, required this.tag})
      : super(key: key);
  final Uint8List image;
  final String tag;
  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        backgroundColor: Colors.black,
        leading: const CloseButton(color: Colors.white),
      ),
      body: Hero(tag: tag, child: PhotoView(imageProvider: MemoryImage(image))),
    );
  }
}

extension FileExt on String {
  Uint8List get bytes => File(this).readAsBytesSync();
  File get file => File(this);
}

class _CardItem extends StatelessWidget {
  const _CardItem({Key? key, required this.child}) : super(key: key);
  final Widget child;
  @override
  Widget build(BuildContext context) {
    return child.box
        .width(double.infinity)
        .withRounded(value: 5)
        .padding(const EdgeInsets.fromLTRB(10, 20, 10, 20))
        .margin(const EdgeInsets.all(10))
        .white
        .make();
  }
}
