import 'package:get/get.dart';
import '../../../configs/config.dart';

class InfoItemUpdate extends StatelessWidget {
  const InfoItemUpdate({
    Key? key,
    required this.title,
    this.content,
    this.innerTextChange,
    this.readOnly = false,
  }) : super(key: key);
  final String title;
  final String? content;
  final bool readOnly;
  final Function(String)? innerTextChange;
  @override
  Widget build(BuildContext context) {
    final controller = TextEditingController()..text = content ?? '';
    final focusNode = FocusNode();
    final titleStyle = context.subtitle1.copyWith(
        color: context.primaryColor, fontWeight: FontWeight.bold, fontSize: 18);
    final contentStyle =
        titleStyle.copyWith(color: Colors.black, fontWeight: FontWeight.normal);
    return Container(
      decoration: BoxDecoration(
          border: Border(
              bottom: BorderSide(width: 1, color: context.dividerColor))),
      margin: const EdgeInsets.symmetric(vertical: 10),
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: <Widget>[
          Container(
            padding: const EdgeInsets.symmetric(horizontal: Dimens.size2),
            child: Text(
              title,
              style: titleStyle,
              maxLines: 2,
            ),
          ),
          Dimens.height10,
          Container(
            child: readOnly
                ? Row(
                    crossAxisAlignment: CrossAxisAlignment.end,
                    children: <Widget>[
                      Expanded(
                        child: Container(
                          margin: const EdgeInsets.only(bottom: Dimens.size10),
                          child: Text(
                            content ?? '',
                            style: contentStyle,
                          ),
                        ),
                      ),
                    ],
                  )
                : Row(
                    children: <Widget>[
                      Expanded(
                        child: Padding(
                          padding: const EdgeInsets.only(bottom: Dimens.size10),
                          child: TextFormField(
                            focusNode: focusNode
                              ..addListener(() {
                                if (!focusNode.hasFocus &&
                                    controller.text != content) {
                                  innerTextChange?.call(controller.text);
                                }
                              }),
                            controller: controller,
                            style: contentStyle,
                            maxLines: 2,
                            minLines: 1,
                            decoration: InputDecoration(
                                counterText: '',
                                hintText: 'empty'.tr,
                                hintStyle: contentStyle.copyWith(
                                  color: const Color.fromRGBO(0, 0, 0, 0.4),
                                ),
                                border: InputBorder.none,
                                isDense: true,
                                contentPadding: const EdgeInsets.all(3.0)),
                          ),
                        ),
                      ),
                    ],
                  ),
          )
        ],
      ),
    );
  }
}
