library liveness;

import 'package:rxdart/rxdart.dart';
import 'dart:async';
import 'dart:io';
import 'dart:typed_data';
import 'package:flutter/cupertino.dart';
import 'package:flutter/foundation.dart';
import 'package:get/get.dart' hide ContextExtensionss;
import 'package:google_ml_kit/google_ml_kit.dart';
import 'dart:math' as math;

import '../../configs/application.dart';
import '../../configs/config.dart';
import '../app/controllers/app_controller.dart';
import '../../routes/app_pages.dart';
import '../../utils/app_utils.dart';
import '../../utils/convert_camera_image_to_png.dart';
import '../../widgets/loading.dart';
part 'widgets/liveness_image_action.dart';
part 'widgets/camera_overlay.dart';
part 'widgets/liveness_title.dart';
part 'widgets/liveness_action.dart';
part 'widgets/liveness_header.dart';
part 'widgets/liveness_process.dart';
part 'enum.dart';

class LivenessDetectionView extends StatefulWidget {
  const LivenessDetectionView({Key? key}) : super(key: key);
  @override
  State<LivenessDetectionView> createState() => _LivenessDetectionViewState();
}

class _LivenessDetectionViewState extends State<LivenessDetectionView> {
  AppController get appController => Get.find();

  final _listenOnDataProcess = PublishSubject<CameraImage>();

  /// This will be stream camera image when once live action is success
  final _listenOnPortraits = PublishSubject<Uint8List>();

  /// This will be stream camera [bool] when face is not recongize
  final _listenWhenOutZone = PublishSubject<bool>();

  /// This will be notification when you allready done all step
  final _isFinish = PublishSubject<bool>();

  /// This will be notification when you on last step
  final _onLastStep = PublishSubject<bool>();

  ValueNotifier<DetectionType> _step = ValueNotifier(DetectionType.EYES_BLINK);
  DetectionDisplay _detectionDisplay = DetectionDisplay.DEFAULT;

  final _listDetectorTypeUsed = <DetectionType>[];
  final _currentStep = ValueNotifier(0);
  final _listMouthProcess = <double>[];
  final _listLeftEyesProcess = <double>[];
  final _listRightEyesProcess = <double>[];
  final int _processLevel = 2;
  final _subsciptions = <StreamSubscription<dynamic>>[];
  final _listDetectorType = <DetectionType>[];

  late DetectionLastStep _detectionLastStep;
  late List<CameraDescription> _cameras;
  late FaceDetector _faceDetector;

  CameraController? _cameraController;
  int _doingStep = 5;
  bool _isBusyProcess = false;
  bool _isOutZone = false;
  bool _isAutoFaceMatching = true;
  @override
  void initState() {
    _faceDetector = GoogleMlKit.vision.faceDetector(const FaceDetectorOptions(
      enableClassification: true,
      enableTracking: true,
      enableLandmarks: true,
      mode: FaceDetectorMode.fast,
    ));
    _isAutoFaceMatching = Application.config.isAutoMatching;
    _doingStep = Application.config.doingStep;
    _detectionLastStep = Application.config.faceConfig.detectionLastStep;
    _detectionDisplay = Application.config.faceConfig.detectionDisplay;
    _listDetectorType.addAll(Application.config.faceConfig.listDetectorType);
    _initStream();
    _initListStep();
    if (_listDetectorTypeUsed.isNotEmpty) {
      _step = ValueNotifier(_listDetectorTypeUsed.first);
      _initFaceDetector();
    }
    super.initState();
  }

  void _initStream() {
    _subsciptions.clear();
    final finisSub = _isFinish.stream.listen((val) {
      if (val == true) {
        Get.offAllNamed(
          Application.config.isSignature
              ? Routes.UPLOAD_SIGNATURE
              : Routes.INFOMATION,
        );
      }
    });
    final dataSub = _listenOnDataProcess.stream.listen((event) {
      appController.imageList.add(event);
    });
    final portraitsSub = _listenOnPortraits.stream.listen((event) {
      appController.portraitsImage = event;
    });
    _subsciptions.addAll([finisSub, dataSub, portraitsSub]);
  }

  Future<void> _initFaceDetector() async {
    _isBusyProcess = false;
    _isOutZone = false;
    _cameras = await availableCameras();
    if (_cameras.isNotEmpty && _cameras.length > 1) {
      _cameraController = CameraController(
        _cameras[1],
        ResolutionPreset.low,
        enableAudio: false,
      );
      await _cameraController?.initialize().then((value) {
        _cameraController?.startImageStream(_processCameraImage);
        setState(() {});
      });
    }
  }

  void _takePortraitsImage({CameraImage? cameraImage}) {
    if (!_isAutoFaceMatching) {
      _cameraController?.takePicture().then((XFile file) {
        var unit8List = File(file.path).readAsBytesSync();
        _listenOnPortraits.sink.add(unit8List);
        _isFinish.sink.add(true);
      });
    } else {
      _listenOnPortraits.sink
          .add(Uint8List.fromList(convertImagetoPng(cameraImage!)!));
    }
  }

  Future _processCameraImage(CameraImage image) async {
    final allBytes = WriteBuffer();
    for (var plane in image.planes) {
      allBytes.putUint8List(plane.bytes);
    }
    final bytes = allBytes.done().buffer.asUint8List();

    final imageSize = Size(image.width.toDouble(), image.height.toDouble());

    final camera = _cameras[1];
    final imageRotation =
        InputImageRotationMethods.fromRawValue(camera.sensorOrientation) ??
            InputImageRotation.Rotation_0deg;

    final inputImageFormat =
        InputImageFormatMethods.fromRawValue(image.format.raw) ??
            InputImageFormat.NV21;

    final planeData = image.planes.map(
      (Plane plane) {
        return InputImagePlaneMetadata(
          bytesPerRow: plane.bytesPerRow,
          height: plane.height,
          width: plane.width,
        );
      },
    ).toList();

    final inputImageData = InputImageData(
      size: imageSize,
      imageRotation: imageRotation,
      inputImageFormat: inputImageFormat,
      planeData: planeData,
    );

    final inputImage =
        InputImage.fromBytes(bytes: bytes, inputImageData: inputImageData);

    await _processImage(inputImage, image);
  }

  Future<void> _processImage(InputImage inputImage, CameraImage image) async {
    if (_isBusyProcess) return;
    final faces = await _faceDetector.processImage(inputImage);
    if (faces.isNotEmpty) {
      if (!_isOutZone) {
        setState(() => _isOutZone = true);
        _listenWhenOutZone.sink.add(false);
      }
      switch (_step.value) {
        case DetectionType.LOOK_LEFT:
          for (var item in faces) {
            if (Platform.isAndroid) {
              if ((item.headEulerAngleY ?? -1) > 30 && _isRecognize(item)) {
                _changeStep(image: image);
              }
            } else {
              if ((item.headEulerAngleY ?? 1) < -30 && _isRecognize(item)) {
                _changeStep(image: image);
              }
            }
          }
          break;
        case DetectionType.LOOK_RIGHT:
          for (var item in faces) {
            if (Platform.isAndroid) {
              if ((item.headEulerAngleY ?? 1) < -30 && _isRecognize(item)) {
                _changeStep(image: image);
              }
            } else {
              if ((item.headEulerAngleY ?? -1) > 30 && _isRecognize(item)) {
                _changeStep(image: image);
              }
            }
          }
          break;
        case DetectionType.EYES_BLINK:
          for (var item in faces) {
            if (_isRecognize(item)) {
              if (_leftEyeProcess(item.leftEyeOpenProbability ?? 1.0, item) ||
                  _rightEyeProcess(item.rightEyeOpenProbability ?? 1.0, item)) {
                _changeStep(image: image);
              }
            }
          }
          break;
        case DetectionType.SMILE:
          for (var item in faces) {
            if (_isRecognize(item) &&
                item.getLandmark(FaceLandmarkType.leftMouth) != null &&
                item.getLandmark(FaceLandmarkType.rightMouth) != null &&
                item.getLandmark(FaceLandmarkType.bottomMouth) != null) {
              if (_mouthProcess(item.smilingProbability ?? 1)) {
                _changeStep(image: image);
              }
            }
          }
          break;
        case DetectionType.LOOK_STRAIGHT:
          for (var item in faces) {
            if (_isRecognize(item)) {
              if ((item.smilingProbability ?? -1) < 0.06 &&
                  ((item.rightEyeOpenProbability ?? 1) > 0.4 &&
                      (item.leftEyeOpenProbability ?? 1) > 0.4) &&
                  (item.headEulerAngleY ?? -1) > -2 &&
                  (item.headEulerAngleY ?? -1) < 2) {
                _changeStep(image: image);
                if (_isAutoFaceMatching) {
                  _takePortraitsImage(cameraImage: image);
                }
              }
            }
          }
          break;
        default:
      }
    } else {
      if (_isOutZone) {
        _initListStep();
        _rollback();
        _cleanListen();
      }
      _isOutZone = false;
    }
  }

  void _rollback() {
    _currentStep.value = 0;
    _step.value = _listDetectorTypeUsed.first;
    if (!_isBusyProcess) {
      _listenWhenOutZone.sink.add(true);
    }
  }

  bool _rightEyeProcess(double rightEyes, Face face) {
    _listRightEyesProcess.add(rightEyes);
    if (_listRightEyesProcess.length < _processLevel) {
      return false;
    } else {
      var _passCount = 0;
      var _oldProcess = 1.0;
      for (var item in _listRightEyesProcess) {
        if (item < _oldProcess) {
          _passCount++;
          _oldProcess = item;
        } else {
          _listRightEyesProcess.removeWhere((element) => element == item);
          break;
        }
      }
      if (_passCount >= _processLevel) {
        if (_listRightEyesProcess.last < 0.018) {
          _listRightEyesProcess.clear();
          return true;
        } else {
          _listRightEyesProcess.clear();
          return false;
        }
      } else {
        return false;
      }
    }
  }

  bool _leftEyeProcess(double leftEyes, Face face) {
    _listLeftEyesProcess.add(leftEyes);
    if (_listLeftEyesProcess.length < _processLevel) {
      return false;
    } else {
      var _passCount = 0;
      var _oldProcess = 1.0;
      for (var item in _listLeftEyesProcess) {
        if (item < _oldProcess) {
          _passCount++;
          _oldProcess = item;
        } else {
          _listLeftEyesProcess.removeWhere((element) => element == item);
          break;
        }
      }
      if (_passCount >= _processLevel) {
        if (_listLeftEyesProcess.last < 0.018) {
          _listLeftEyesProcess.clear();
          return true;
        } else {
          _listLeftEyesProcess.clear();
          return false;
        }
      } else {
        return false;
      }
    }
  }

  void _changeStep({required CameraImage image}) {
    setState(() => _isBusyProcess = true);
    if (_currentStep.value <= _listDetectorTypeUsed.length) {
      _currentStep.value++;
      if (_currentStep.value < _listDetectorTypeUsed.length) {
        _step.value = _listDetectorTypeUsed[_currentStep.value];
        if (_isAutoFaceMatching && _step.value == DetectionType.LOOK_STRAIGHT) {
          Future.delayed(const Duration(seconds: 1)).whenComplete(
            () {
              setState(() => _isBusyProcess = false);
            },
          );
        } else {
          setState(() => _isBusyProcess = false);
        }
      } else if (_currentStep.value == _listDetectorTypeUsed.length) {
        _cameraController?.stopImageStream();
        if (_detectionLastStep != DetectionLastStep.NONE &&
            !_isAutoFaceMatching) {
          _onLastStep.sink.add(true);
        } else {
          if (_isAutoFaceMatching) {
            Future.delayed(const Duration(seconds: 3)).whenComplete(
              () => _isFinish.sink.add(true),
            );
          } else {
            _isFinish.sink.add(true);
          }
        }
      }
    }
    _listenOnDataProcess.sink.add(image);
  }

  bool _mouthProcess(double smilling) {
    _listMouthProcess.add(smilling);
    if (_listMouthProcess.length < _processLevel) {
      return false;
    } else {
      var _passCount = 0;
      var _oldProcess = -0.1;
      for (var item in _listMouthProcess) {
        if (item > _oldProcess) {
          _passCount++;
          _oldProcess = item;
        } else {
          _listMouthProcess.removeWhere((element) => element == item);
          break;
        }
      }
      if (_passCount >= _processLevel) {
        if (_listMouthProcess.last > 0.75) {
          _listMouthProcess.clear();
          return true;
        } else {
          _listMouthProcess.clear();
          return false;
        }
      } else {
        return false;
      }
    }
  }

  bool _isRecognize(Face face) {
    if (face.boundingBox.left > -11 &&
        face.boundingBox.right < 350 &&
        face.boundingBox.top > 30 &&
        face.boundingBox.bottom < 300) {
      if (face.headEulerAngleY != null &&
          face.trackingId != null &&
          face.headEulerAngleZ != null &&
          face.rightEyeOpenProbability != null &&
          face.smilingProbability != null) {
        return true;
      } else {
        return false;
      }
    } else {
      return false;
    }
  }

  void _initListStep() {
    _listDetectorTypeUsed.clear();
    var promissedAction = DetectionType.LOOK_STRAIGHT;
    var listImportant = [DetectionType.EYES_BLINK, DetectionType.SMILE];
    listImportant.shuffle();
    if (_detectionDisplay == DetectionDisplay.RANDOM) {
      _listDetectorType.shuffle();
    }
    _listDetectorTypeUsed.addAll(_listDetectorType.take(_doingStep).toList());
    if (_listDetectorTypeUsed
        .where((element) =>
            element == DetectionType.EYES_BLINK ||
            element == DetectionType.SMILE)
        .isEmpty) {
      var i = math.Random().nextInt(_listDetectorTypeUsed.length);
      _listDetectorTypeUsed.removeAt(i);
      _listDetectorTypeUsed.add(listImportant.first);
    }
    if (!_listDetectorTypeUsed.contains(promissedAction)) {
      for (var item in _listDetectorTypeUsed) {
        if (!listImportant.contains(item)) {
          _listDetectorTypeUsed.remove(item);
          _listDetectorTypeUsed.add(promissedAction);
          break;
        }
      }
    }

    if (_detectionDisplay == DetectionDisplay.RANDOM) {
      _listDetectorTypeUsed.shuffle();
    }
  }

  @override
  Widget build(BuildContext context) {
    final sizeWidth = context.width;
    final sizeHeight = context.height * 2 / 3;
    return WillPopScope(
      onWillPop: AppUtils.onWillPop,
      child: Scaffold(
        backgroundColor: context.primaryVariant,
        body: SafeArea(
          child: Padding(
            padding: const EdgeInsets.fromLTRB(20, 20, 20, 0),
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.stretch,
              mainAxisAlignment: MainAxisAlignment.center,
              children: [
                LivenessHeader(onLastStep: _onLastStep.stream),
                Expanded(
                  flex: 12,
                  child: Stack(
                    alignment: Alignment.center,
                    children: [
                      Padding(
                        padding: const EdgeInsets.all(1),
                        child: _cameraController != null &&
                                _cameraController!.value.isInitialized
                            ? CameraPreview(_cameraController!)
                            : const Loading(),
                      ),
                      CameraOverlay(
                        colors: context.primaryVariant,
                        width: sizeWidth,
                        height: sizeHeight,
                      ),
                      LivenessProcesss(
                        sizeHeight: sizeHeight,
                        sizeWidth: sizeWidth,
                        currentStep: _currentStep,
                        doingStep: _doingStep,
                      ),
                    ],
                  ),
                ),
                LivenessAction(
                  onLastStep: _onLastStep.stream,
                  takePortraitsImage: _takePortraitsImage,
                  elseChild: LivenessTitle(
                    listenWhenOutZone: _listenWhenOutZone.stream,
                    step: _step,
                  ),
                )
              ],
            ),
          ),
        ),
      ),
    );
  }

  @override
  void dispose() {
    _cameraController?.dispose();
    _disposeListen();
    _subsciptions.forEach((e) => e.cancel());
    super.dispose();
  }

  void _cleanListen() {
    _onLastStep.done;
    _isFinish.done;
    _listenOnPortraits.done;
    _listenOnDataProcess.done;
    _listenWhenOutZone.done;
  }

  void _disposeListen() {
    _listenOnDataProcess.close();
    _listenOnPortraits.close();
    _listenWhenOutZone.close();
    _isFinish.close();
    _onLastStep.close();
  }
}
