import 'package:flutter_colorpicker/flutter_colorpicker.dart';
import 'package:get/get.dart' hide ContextExtensionss;

import '../../../configs/config.dart';
import '../../../widgets/app_button.dart';
import '../controllers/setting_controller.dart';

class SettingView extends GetView<SettingController> {
  const SettingView({Key? key}) : super(key: key);

  void changeColorTitle(Color val, BuildContext context) {
    AppTheme.onBackground = val;
    AppTheme.theme = AppTheme.theme
        .copyWith(colorScheme: context.colorScheme.copyWith(onBackground: val));
    Get.changeTheme(AppTheme.theme);
  }

  void changePrimaryColor(Color val, BuildContext context) {
    AppTheme.primaryColor = val;
    AppTheme.theme = AppTheme.theme.copyWith(
        scaffoldBackgroundColor: val,
        primaryColor: val,
        appBarTheme: context.appBarTheme.copyWith(color: val));
    Get.changeTheme(AppTheme.theme);
  }

  @override
  Widget build(BuildContext context) {
    Widget header(String title) =>
        title.text.size(18).bold.make().pSymmetric(h: 20, v: 15);
    return Scaffold(
      appBar: AppBar(
        title: 'Cài đặt'.tr.text.color(context.onBackground).make(),
        centerTitle: true,
      ),
      body: Obx(
        () => SafeArea(
          child: Container(
            color: Colors.white,
            child: Column(
              children: [
                Expanded(
                  child: SingleChildScrollView(
                    child: Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                      children: [
                        header('Màu'),
                        Row(
                          mainAxisAlignment: MainAxisAlignment.spaceAround,
                          children: [
                            Column(
                              children: [
                                'Background'.text.bold.make(),
                                Dimens.height5,
                                ColorButton(
                                    activeColor: context.primaryColor,
                                    onColorChanged: (val) {
                                      changePrimaryColor(val, context);
                                    })
                              ],
                            ),
                            Column(
                              children: [
                                'On Background'.text.bold.make(),
                                Dimens.height5,
                                ColorButton(
                                  activeColor: context.onBackground,
                                  onColorChanged: (val) {
                                    changeColorTitle(val, context);
                                  },
                                )
                              ],
                            )
                          ],
                        ),
                        header('Thông tin chung'),
                        CheckboxListTile(
                          activeColor: context.primaryColor,
                          value: controller.config.value.isHistoy,
                          controlAffinity: ListTileControlAffinity.leading,
                          title: 'Hiển thị lịch sử'.tr.text.make(),
                          onChanged: (value) {
                            controller.config.update((val) {
                              val!.isHistoy = value!;
                            });
                          },
                        ),
                        CheckboxListTile(
                          activeColor: context.primaryColor,
                          value: controller.config.value.isShowInfo,
                          controlAffinity: ListTileControlAffinity.leading,
                          title: 'Hiển thị thông tin OCR'.tr.text.make(),
                          onChanged: (value) {
                            controller.config.update((val) {
                              val!.isShowInfo = value!;
                            });
                          },
                        ),
                        CheckboxListTile(
                          activeColor: context.primaryColor,
                          value: controller.config.value.isSignature,
                          controlAffinity: ListTileControlAffinity.leading,
                          title: 'Hiển thị tải lên chữ ký'.tr.text.make(),
                          onChanged: (value) {
                            controller.config.update((val) {
                              val!.isSignature = value!;
                            });
                          },
                        ),
                        CheckboxListTile(
                          activeColor: context.primaryColor,
                          value: controller.config.value.readOnlyInfo,
                          controlAffinity: ListTileControlAffinity.leading,
                          title: 'Chỉnh sửa thông tin'.tr.text.make(),
                          onChanged: (value) {
                            controller.config.update((val) {
                              val!.readOnlyInfo = value!;
                            });
                          },
                        ),
                        CheckboxListTile(
                          activeColor: context.primaryColor,
                          value: controller.config.value.isAutoMatching,
                          controlAffinity: ListTileControlAffinity.leading,
                          title: 'Auto matching face detection'.tr.text.make(),
                          onChanged: (value) {
                            controller.config.update((val) {
                              val!.isAutoMatching = value!;
                            });
                          },
                        ),
                        CheckboxListTile(
                          activeColor: context.primaryColor,
                          value: controller.config.value.isSaveCapture,
                          controlAffinity: ListTileControlAffinity.leading,
                          title: 'Lưu hình ảnh đã quét vào thư viện'
                              .tr
                              .text
                              .make(),
                          onChanged: (value) {
                            controller.config.update((val) {
                              val!.isSaveCapture = value!;
                            });
                          },
                        ),
                        // CheckboxListTile(
                        //   value: controller.config.value.faceConfig.detectionDisplay == DetectionDisplay.RANDOM,
                        //   controlAffinity: ListTileControlAffinity.leading,
                        //   title: 'Nhận diện khuôn mặt ngẫu nhiên'.tr.text.make(),
                        //   onChanged: (value) {
                        //     controller.config.update((val) {
                        //       val!.faceConfig.detectionDisplay = controller.config.value.faceConfig.detectionDisplay == DetectionDisplay.RANDOM ? DetectionDisplay.DEFAULT : DetectionDisplay.RANDOM;
                        //     });
                        //   },
                        // ),
                        header('Thông tin OCR'),
                        CheckboxListTile(
                          value: controller.config.value.autoORC,
                          controlAffinity: ListTileControlAffinity.leading,
                          title: 'Tự động scan OCR sau 5s'.tr.text.make(),
                          onChanged: (value) {
                            controller.config.update((val) {
                              val!.autoORC = value!;
                            });
                          },
                        ),
                        header('Nhận diện khuôn mặt theo'),
                        const DropdownButtonCustom(hintText: 'hintText'),
                      ],
                    ),
                  ),
                ),
                Dimens.height20,
                AppButton(
                  'Confirm'.tr,
                  fontSize: 18,
                  borderRadius: 30,
                  onPressed: controller.saveConfig,
                ).box.size(220, 50).make(),
                Dimens.height20,
              ],
            ),
          ),
        ),
      ),
    );
  }
}

class DropdownButtonCustom extends StatelessWidget {
  const DropdownButtonCustom({Key? key, required this.hintText})
      : super(key: key);
  final String hintText;
  SettingController get to => Get.find();
  @override
  Widget build(BuildContext context) {
    return Obx(
      () => DropdownButtonHideUnderline(
        child: Container(
          padding: const EdgeInsets.symmetric(horizontal: 10),
          decoration: const ShapeDecoration(
            shape: RoundedRectangleBorder(
              side: BorderSide(style: BorderStyle.solid, color: Colors.grey),
              borderRadius: BorderRadius.all(Radius.circular(5.0)),
            ),
          ),
          child: DropdownButton<dynamic>(
            isExpanded: true,
            value: to.config.value.faceConfig.doingStep,
            hint: hintText.text
                .size(12)
                .color(const Color(0xff9B9B9B))
                .make()
                .pOnly(left: 10, right: 10),
            items: [
              DropdownMenuItem<dynamic>(
                value: 2,
                child: '2 ${'step'.tr}'.text.size(15).make(),
              ),
              DropdownMenuItem<dynamic>(
                value: 3,
                child: "3 ${'step'.tr}".text.size(15).make(),
              ),
              DropdownMenuItem<dynamic>(
                value: 4,
                child: '4 ${'step'.tr}'.text.size(15).make(),
              ),
              DropdownMenuItem<dynamic>(
                value: 5,
                child: '5 ${'step'.tr}'.text.size(15).make(),
              )
            ],
            onChanged: (value) {
              to.config.update((val) {
                val!.faceConfig.doingStep = value;
              });
            },
          ),
        ),
      ).pSymmetric(h: 20),
    );
  }
}

class ColorButton extends StatelessWidget {
  const ColorButton(
      {Key? key, required this.activeColor, required this.onColorChanged})
      : super(key: key);
  final Color activeColor;
  final ValueChanged<Color> onColorChanged;
  @override
  Widget build(BuildContext context) {
    return VxBox()
        .size(50, 50)
        .color(activeColor)
        .roundedFull
        .shadow2xl
        .make()
        .onTap(() {
      Get.dialog(
        Dialog(
          child: SingleChildScrollView(
            child: Column(
              children: [
                ColorPicker(
                  pickerColor: activeColor,
                  onColorChanged: onColorChanged,
                )
              ],
            ),
          ),
        ),
      );
    });
  }
}
