import 'dart:io';
import 'package:flutter/cupertino.dart';
import 'package:get/get.dart';
import 'package:image_picker/image_picker.dart';
import 'package:pedantic/pedantic.dart';
import 'package:signature/signature.dart';

import '../../../configs/config.dart';
import '../../app/controllers/app_controller.dart';
import '../../../routes/app_pages.dart';
import '../../../utils/app_utils.dart';
import '../../../utils/logger.dart';
import '../../../widgets/camera/camera_picker.dart';

class UploadSignatureController extends GetxController {
  final controller = SignatureController(
    penStrokeWidth: 2,
    penColor: Colors.black,
    exportBackgroundColor: Colors.white,
  );

  final ImagePicker _picker = ImagePicker();
  final RxString _image = RxString('');

  String get image => _image.value;
  @override
  void onInit() {
    super.onInit();
  }

  @override
  void onReady() {
    super.onReady();
  }

  @override
  void onClose() {}
  void showBottomSheet() {
    Get.bottomSheet(
      CupertinoActionSheet(
        actions: <CupertinoActionSheetAction>[
          CupertinoActionSheetAction(
            onPressed: () => pickerImage(ImageSource.gallery),
            child: Text('photo_gallary'.tr),
          ),
          CupertinoActionSheetAction(
            onPressed: () => pickerImage(ImageSource.camera),
            child: const Text('Camera'),
          )
        ],
        cancelButton: CupertinoActionSheetAction(
          onPressed: () => Get.back(),
          child: Text('cancel'.tr),
        ),
      ),
    );
  }

  void clearImage() {
    _image.value = '';
  }

  Future<void> pickerImage(ImageSource source) async {
    Get.back();
    try {
      if (source == ImageSource.camera) {
        final file = await CameraPicker.pickFromCamera(
          Get.context!,
          enableRecording: false,
        );
        if (file != null) {
          _image.value = (await file.file)!.path;
        }
      } else {
        final file = await _picker.pickImage(source: source, maxWidth: 720);
        _image.value = file?.path ?? '';
      }
    } catch (e) {
      Get.snackbar('Error', e.toString());
      UtilLogger.log(e);
    }
  }

  Future<void> captureSignture() async {
    var uint8list = await controller.toPngBytes();
    Get.find<AppController>().signature = uint8list;
    AppUtils.saveImage(uint8list);
    unawaited(Get.offNamed(Routes.INFOMATION));
  }

  void confirm() {
    Get.find<AppController>().signature = File(image).readAsBytesSync();
    AppUtils.saveImage(File(image).readAsBytesSync());
    Get.toNamed(Routes.INFOMATION);
  }
}
