import 'package:flutter/cupertino.dart';
import 'package:get/get.dart' hide ContextExtensionss;

import '../../../configs/config.dart';
import '../../../routes/app_pages.dart';
import '../../../widgets/app_button.dart';
import '../../../widgets/page_indicators.dart';
import '../controllers/welcome_controller.dart';

class WelcomeView extends GetView<WelcomeController> {
  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: Colors.white,
      appBar: AppBar(
        leading: const BackButton(color: Colors.black),
        backgroundColor: Colors.transparent,
        elevation: 0,
        title: Dimens.logo,
        centerTitle: true,
      ),
      body: <Widget>[
        [
          PageView.builder(
            controller: controller.pageController,
            itemCount: controller.slides.length,
            onPageChanged: (index) {
              controller.indexPage.value = index;
            },
            itemBuilder: (BuildContext context, int index) {
              return ItemPage(slider: controller.slides[index]);
            },
          ).h(context.height * .5),
          Obx(() => PageIndicators(
                numberIndicators: controller.slides.length,
                currentIndex: controller.indexPage.value,
              )).pOnly(
            top: context.height * .04,
            bottom: context.height * .04,
          )
        ].column(alignment: MainAxisAlignment.center).expand(),
        AppButton(
          'start'.tr,
          borderRadius: 30,
          fontSize: 18,
          onPressed: () => Get.offNamed(Routes.CHOOSE_SCREEN),
        ).box.size(220, 50).make().pOnly(bottom: context.height * .04)
      ].column().safeArea(),
    );
  }
}

class ItemPage extends StatelessWidget {
  const ItemPage({
    Key? key,
    required this.slider,
  }) : super(key: key);
  final SlideModel slider;

  @override
  Widget build(BuildContext context) {
    const width = 414;
    const height = 896;
    return <Widget>[
      Image.asset(
        slider.image,
        width: context.width * 283 / width,
        height: context.height * 280 / height,
      ),
      <Widget>[
        slider.title.tr.text
            .size(18)
            .maxLines(3)
            .center
            .make()
            .pSymmetric(h: 15),
      ].column().pOnly(top: context.height * 60 / height)
    ].column();
  }
}
