import 'dart:typed_data';
import 'package:get/get.dart';
import 'package:photo_manager/photo_manager.dart';

import '../configs/application.dart';
import '../configs/config.dart';
import '../routes/app_pages.dart';
import 'logger.dart';

class AppUtils {
  factory AppUtils() {
    return _instance;
  }

  AppUtils._internal();

  static final AppUtils _instance = AppUtils._internal();
  static Future<bool> onWillPop() async {
    await Get.dialog(const BackDialog());
    return false;
  }

  static void saveImage(Uint8List? data) {
    try {
      if (data != null && Application.config.isSaveCapture) {
        PhotoManager.editor.saveImage(data);
      }
    } catch (e) {
      UtilLogger.log(e);
    }
  }
}

class BackDialog extends StatelessWidget {
  const BackDialog({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Dialog(
      shape: RoundedRectangleBorder(borderRadius: BorderRadius.circular(10)),
      child: SizedBox(
        width: 300,
        child: Column(
          mainAxisSize: MainAxisSize.min,
          children: [
            Padding(
              padding: const EdgeInsets.symmetric(horizontal: 20, vertical: 50),
              child: 'Bạn có chắc chắn muốn dừng không?'
                  .tr
                  .text
                  .center
                  .size(16)
                  .makeCentered(),
            ),
            Divider(height: 0, color: context.dividerColor),
            Row(children: [
              'Yes'
                  .tr
                  .text
                  .white
                  .make()
                  .box
                  .alignCenter
                  .padding(const EdgeInsets.all(10))
                  .withDecoration(BoxDecoration(
                    color: context.primaryColor,
                    borderRadius: const BorderRadius.only(
                      bottomLeft: Radius.circular(10),
                    ),
                  ))
                  .make()
                  .onTap(() {
                Get.back();
                Get.offAllNamed(Routes.CHOOSE_SCREEN);
              }).expand(),
              'No'
                  .tr
                  .text
                  .color(context.primaryColor)
                  .make()
                  .box
                  .alignCenter
                  .padding(const EdgeInsets.all(10))
                  .make()
                  .onTap(Get.back)
                  .expand(),
            ])
          ],
        ),
      ),
    );
  }
}
