import 'dart:math' as math;
import 'package:image/image.dart' as imglib;
import 'dart:io';
import 'dart:typed_data';
import '../configs/config.dart';
import 'logger.dart';

const shift = (0xFF << 24);
List<int>? convertImagetoPng(CameraImage image) {
  try {
    late imglib.Image img;
    if (image.format.group == ImageFormatGroup.yuv420) {
      img = _convertYUV420(image);
    } else if (image.format.group == ImageFormatGroup.bgra8888) {
      img = _convertBGRA8888(image);
    }

    var pngEncoder = imglib.PngEncoder(level: 0, filter: 0);

    var png = pngEncoder.encodeImage(img);
    if (Platform.isAndroid) {
      var detail = imglib.decodeImage(png);
      var width = detail!.width;
      var height = detail.height;
      if (height < width) {
        var fixedImage = imglib.copyRotate(detail, -90);
        png = imglib.encodePng(fixedImage);
      }
    }
    return png;
  } catch (e) {
    UtilLogger.log('>>>>>>>>>>>> ERROR:' + e.toString());
  }
  return null;
}

// CameraImage BGRA8888 -> PNG

// Color
imglib.Image _convertBGRA8888(CameraImage image) {
  return imglib.Image.fromBytes(
    image.width,
    image.height,
    image.planes[0].bytes,
    format: imglib.Format.bgra,
  );
}

// CameraImage YUV420_888 -> PNG -> Image (compresion:0, filter: none)
// Black

Uint8List? convertJpeg(CameraImage image) {
  UtilLogger.log(image.planes[0].bytes.toString());
  var img = imglib.Image.fromBytes(
      image.width, image.height, image.planes[0].bytes,
      format: imglib.Format.rgb, channels: imglib.Channels.rgb);
  return img.getBytes();
}

imglib.Image _convertYUV420(CameraImage image) {
  final width = image.width;
  final height = image.height;
  final uvRowStride = image.planes[1].bytesPerRow;
  final uvPixelStride = image.planes[1].bytesPerPixel;

  var img = imglib.Image(width, height);
  for (var x = 0; x < width; x++) {
    for (var y = 0; y < height; y++) {
      var uvIndex =
          uvPixelStride! * (x / 2).floor() + uvRowStride * (y / 2).floor();
      final index = y * width + x;

      final yp = image.planes[0].bytes[index];
      final up = image.planes[1].bytes[uvIndex];
      final vp = image.planes[2].bytes[uvIndex];
      var r = (yp + vp * 1436 / 1024 - 179).round().clamp(0, 255);
      var g = (yp - up * 46549 / 131072 + 44 - vp * 93604 / 131072 + 91)
          .round()
          .clamp(0, 255);
      var b = (yp + up * 1814 / 1024 - 227).round().clamp(0, 255);
      img.data[index] = shift | (b << 16) | (g << 8) | r;
    }
  }

  return img;
}

String formatBytes(int bytes, int decimals) {
  if (bytes <= 0) {
    return '0 B';
  }
  const suffixes = ['B', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB'];
  final i = (math.log(bytes) / math.log(1024)).floor();
  return ((bytes / math.pow(1024, i)).toStringAsFixed(decimals)) +
      ' ' +
      suffixes[i];
}
