import 'dart:ffi';
import 'dart:io';
import 'dart:typed_data';
import 'package:camera/camera.dart';
import 'package:ffi/ffi.dart';
import 'package:image/image.dart' as imglib;
import 'package:flutter_cache_manager/flutter_cache_manager.dart';

typedef ConvertFunc = Pointer<Uint32> Function(Pointer<Uint8>, Pointer<Uint8>, Pointer<Uint8>, Int32, Int32, Int32, Int32);
typedef Convert = Pointer<Uint32> Function(Pointer<Uint8>, Pointer<Uint8>, Pointer<Uint8>, int, int, int, int);

class FileUtils {
  // static final DynamicLibrary convertImageLib = Platform.isAndroid ? DynamicLibrary.open('libconvertImage.so') : DynamicLibrary.process();
  static late DefaultCacheManager manager;
  static late Convert conv;
  factory FileUtils() {
    return _instance;
  }

  FileUtils._internal();
  static void init() {
    manager = DefaultCacheManager();
    // conv = convertImageLib.lookup<NativeFunction<ConvertFunc>>('convertImage').asFunction<Convert>();
  }

  static final FileUtils _instance = FileUtils._internal();

  static Future<String> saveByteFile(Uint8List? uint8list) async {
    if (uint8list == null) {
      return '';
    }
    final url = DateTime.now().microsecondsSinceEpoch.toString();
    final file = await manager.putFile(url, uint8list);
    return file.path;
  }

  static Future<String> getFileCache(String url) async {
    final fileInfo = await manager.getFileFromCache(url);
    if (fileInfo != null) {
      return fileInfo.file.path;
    }
    return '';
  }

  static Uint8List formatImage(CameraImage cameraImage) {
    late imglib.Image img;
    if (Platform.isAndroid) {
      final p = calloc.allocate(cameraImage.planes[0].bytes.length) as Pointer<Uint8>;
      final p1 = calloc.allocate(cameraImage.planes[1].bytes.length) as Pointer<Uint8>;
      final p2 = calloc.allocate(cameraImage.planes[2].bytes.length) as Pointer<Uint8>;
      final pointerList = p.asTypedList(cameraImage.planes[0].bytes.length);
      final pointerList1 = p1.asTypedList(cameraImage.planes[1].bytes.length);
      final pointerList2 = p2.asTypedList(cameraImage.planes[2].bytes.length);
      pointerList.setRange(0, cameraImage.planes[0].bytes.length, cameraImage.planes[0].bytes);
      pointerList1.setRange(0, cameraImage.planes[1].bytes.length, cameraImage.planes[1].bytes);
      pointerList2.setRange(0, cameraImage.planes[2].bytes.length, cameraImage.planes[2].bytes);
      final imgP = conv(p, p1, p2, cameraImage.planes[1].bytesPerRow, cameraImage.planes[1].bytesPerPixel!, cameraImage.planes[0].bytesPerRow, cameraImage.height);
      var imgData = imgP.asTypedList((cameraImage.planes[0].bytesPerRow * cameraImage.height));
      img = imglib.Image.fromBytes(cameraImage.height, cameraImage.planes[0].bytesPerRow, imgData);
      calloc.free(p);
      calloc.free(p1);
      calloc.free(p2);
      calloc.free(imgP);
    } else if (Platform.isIOS) {
      img = imglib.Image.fromBytes(
        cameraImage.planes[0].bytesPerRow,
        cameraImage.height,
        cameraImage.planes[0].bytes,
        format: imglib.Format.bgra,
      );
    }
    return Uint8List.fromList(imglib.encodePng(img));
  }
}
