import '../configs/config.dart';

enum ButtonType { normal, outline, round, roundOutline, text }

class AppButton extends StatelessWidget {
  const AppButton(
    this.text, {
    Key? key,
    this.onPressed,
    this.icon,
    this.loading = false,
    this.disabled = false,
    this.type = ButtonType.normal,
    this.borderRadius = 10.0,
    this.borderWidth = 1,
    this.borderColor,
    this.textColor,
    this.color,
    this.elevation = 0,
    this.fontSize = 14,
    this.axisSize = MainAxisSize.max,
  }) : super(key: key);
  final VoidCallback? onPressed;
  final String text;
  final bool loading;
  final bool disabled;
  final ButtonType type;
  final Widget? icon;
  final Color? borderColor, textColor, color;
  final double borderRadius, borderWidth, elevation, fontSize;
  final MainAxisSize axisSize;
  Widget _buildLoading() {
    if (!loading) {
      return Container();
    }
    return Container(
      margin: const EdgeInsets.only(left: 8, right: 8),
      width: 14,
      height: 14,
      child: const CircularProgressIndicator(
        strokeWidth: 1.5,
        valueColor: AlwaysStoppedAnimation(Colors.white),
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    final label = <Widget>[
      text.text.size(fontSize).color(textColor ?? Colors.white).make(),
      _buildLoading()
    ].row(
      crossAlignment: CrossAxisAlignment.center,
      alignment: MainAxisAlignment.center,
      axisSize: axisSize,
    );
    switch (type) {
      case ButtonType.outline:
        Widget button = MaterialButton(
          color: color,
          elevation: elevation,
          shape: RoundedRectangleBorder(
            borderRadius: BorderRadius.circular(borderRadius),
            side: BorderSide(
              width: borderWidth,
              color: borderColor ?? context.primaryColor,
            ),
          ),
          onPressed: onPressed,
          child: text.text
              .buttonText(context)
              .size(fontSize)
              .color(textColor ?? context.button.color!)
              .make(),
        );
        if (icon != null) {
          button = OutlinedButton.icon(
            onPressed: disabled ? null : onPressed,
            icon: icon!,
            label: label,
          );
        }

        return button;

      case ButtonType.round:
        Widget button = ElevatedButton(
          onPressed: disabled ? null : onPressed,
          child: label,
        );
        if (icon != null) {
          button = ElevatedButton.icon(
            onPressed: disabled ? null : onPressed,
            icon: icon!,
            label: label,
          );
        }

        return button;

      case ButtonType.roundOutline:
        Widget button = OutlinedButton(
          onPressed: disabled ? null : onPressed,
          child: label,
        );
        if (icon != null) {
          button = OutlinedButton.icon(
            onPressed: disabled ? null : onPressed,
            icon: icon!,
            label: label,
          );
        }

        return button;

      case ButtonType.text:
        Widget button = TextButton(
          onPressed: disabled ? null : onPressed,
          child: label,
        );
        if (icon != null) {
          button = TextButton.icon(
            onPressed: disabled ? null : onPressed,
            icon: icon!,
            label: label,
          );
        }

        return button;
      default:
        Widget button = MaterialButton(
          elevation: elevation,
          shape: RoundedRectangleBorder(
            borderRadius: BorderRadius.circular(borderRadius),
          ),
          color: color ?? context.primaryColor,
          onPressed: disabled ? null : onPressed,
          child: Row(
            crossAxisAlignment: CrossAxisAlignment.center,
            mainAxisAlignment: MainAxisAlignment.center,
            mainAxisSize: axisSize,
            children: <Widget>[
              if (icon != null) icon!.pOnly(right: 5),
              text.text
                  .buttonText(context)
                  .size(fontSize)
                  .color(textColor ?? context.onBackground)
                  .make(),
              _buildLoading(),
            ],
          ),
        );
        // if (icon != null) {
        //   button = ElevatedButton.icon(
        //     onPressed: disabled ? null : onPressed,
        //     icon: icon!,
        //     label: Row(
        //       crossAxisAlignment: CrossAxisAlignment.center,
        //       mainAxisAlignment: MainAxisAlignment.center,
        //       mainAxisSize: axisSize,
        //       children: <Widget>[
        //         text.text.buttonText(context).size(fontSize).color(Colors.white).make(),
        //         _buildLoading(),
        //       ],
        //     ),
        //   );
        // }

        return button;
    }
  }
}
