import '../configs/config.dart';

class AppListTitle extends StatelessWidget {
  const AppListTitle({
    Key? key,
    this.title = '',
    this.subtitle,
    this.trailing,
    this.icon,
    this.onPressed,
    this.border = false,
    this.titleStyle,
    this.subtitleStyle,
  }) : super(key: key);
  final String title;
  final String? subtitle;
  final Widget? trailing;
  final Widget? icon;
  final VoidCallback? onPressed;
  final bool border;
  final TextStyle? titleStyle, subtitleStyle;
  @override
  Widget build(BuildContext context) {
    Border? borderWidget;
    Widget subTitle = Container();
    Widget iconWidget = Dimens.empty;
    if (icon != null) {
      iconWidget = icon!.pOnly(left: 16, right: 16);
    }
    if (subtitle != null) {
      subTitle = (subtitle ?? '')
          .text
          .textStyle(subtitleStyle ?? context.caption)
          .overflow(TextOverflow.ellipsis)
          .maxLines(2)
          .make()
          .pOnly(top: 8);
    }
    if (border) {
      borderWidget = Border(
        bottom: BorderSide(
          width: 1,
          color: context.dividerColor,
        ),
      );
    }
    return <Widget>[
      iconWidget,
      Container(
        decoration: BoxDecoration(border: borderWidget),
        child: Row(
          crossAxisAlignment: CrossAxisAlignment.center,
          children: <Widget>[
            <Widget>[
              title.text
                  .textStyle(titleStyle ??
                      context.bodyText1.copyWith(fontWeight: FontWeight.w400))
                  .maxLines(2)
                  .overflow(TextOverflow.ellipsis)
                  .make(),
              subTitle
            ]
                .column(crossAlignment: CrossAxisAlignment.start)
                .pOnly(top: 10, bottom: 10)
                .expand(),
            (trailing ?? Container()).pOnly(left: 16, right: 16)
          ],
        ),
      ).expand(),
    ].row(axisSize: MainAxisSize.max).onInkTap(() {
      if (onPressed != null) {
        onPressed?.call();
      }
    });
  }
}
